package projekt.enigma.database;

import java.sql.*;

/**
 * Dies ist die SQLite Datenbank Klasse
 * In der SQLite Datei im Resources Ordner liegt das Codebuch welches benötigt wird um die
 * Nachrichten mit der Enigma zu ver/entschlüsseln.
 * Die Datenbank ist statisch und deshalb konnte sie im Resources Ordner hinterlegt werden.
 * Für jeden Tag des Monats (1-31) ist ein eigener Eintrag enthalten welcher über die Funktion
 * getCodebuch abgerufen werden kann.
 * Zurückgegeben wird ein Array welcher in der getCodebuch Funktion genauer dokumentiert ist.
 */
public class DatenbankSqlite {

	/**
	 * Verbindung zur SQLite Datenbank aufbauen.
	 * Die Datei liegt in den Resources und ändert sich nicht,
	 * weshalb der Pfad zu derselben fest hinterlegt ist.
	 *
	 * @return conn : Connection
	 */
	private Connection connect() {

		String url = "jdbc:sqlite::resource:projekt/enigma/codebuch.sqlite";
		Connection conn = null;

		try {
			conn = DriverManager.getConnection(url);
		} catch (SQLException e) {
			e.printStackTrace();
		}

		return conn;
	}

	/**
	 * Liest das Codebuch aus der SQLite Datenbank aus
	 * Dieser Funktion muss ein tag als int Wert übergeben werden.
	 * Anschließend wir eine Verbindung zur SQLite Datenbank aufgebaut und das Codebuch angefragt.
	 * Dieses wird in ein Object vom Type Codebuch gepackt und zurück gegeben.
	 *
	 * Hole den heutigen Eintrag aus der SQLite Datenbank und erstelle daraus ein Codebuch Objekt
	 *
	 * @param tag : int : Tag für welchen ein Codebuch benötigt wird.
	 * @return String : Die Einstellungen des angefragten Tages
	 * <p>
	 *     String[0] : Tag
	 *     String[1] : Walzenlage
	 *     String[2] : Ringstellung
	 *     String[3] : Steckverbindung
	 * </p>
	 */
	public String[] getCodebuch(int tag) {

		String[] codebuch = new String[4];

		try {
			Connection conn = this.connect();
			PreparedStatement pstm = conn.prepareStatement("SELECT * FROM table_codebuch WHERE Tag=? LIMIT 1");
			pstm.setInt(1, tag);
			ResultSet rs = pstm.executeQuery();

			while (rs.next()) {
				codebuch[0] = rs.getString("Tag");
				codebuch[1] = rs.getString("Walzenlage");
				codebuch[2] = rs.getString("Ringstellung");
				codebuch[3] = rs.getString("Steckverbindung");
			}
		} catch (SQLException ignored) {
		}

		return codebuch;
	}
}
